@@ -1,34 +1,44 @@
 package Mojolicious::Che;
 use Mojo::Base::Che 'Mojolicious';
-#~ use Mojo::Loader qw(load_class);
-
-our $VERSION = '0.024';
-
-=pod
-
-=head1 VERSION
-
-0.024
-
-=cut
+use Mojo::Log::Che;
+use Mojo::Loader qw(load_class);
 
 sub new {
-  my $app = shift->SUPER::new;
-  my %args = @_;
-  $app->plugin(Config =>{file => $args{config} || 'Config.pm'});
+  my ($class, %args) = @_;
+  my $config = delete($args{config}) || 'Config.pm';
+  my $app = $class->SUPER::new(%args);
+  
+  $app->plugin(Config =>{file => $config});
   my $conf = $app->config;
   
-  my $defaults = $conf->{'mojo_defaults'} || $conf->{'mojo'}{'defaults'};
+  my $defaults = $conf->{'mojo_defaults'} || $conf->{'mojo'}{'defaults'}  || $conf->{'mojo.defaults'};
   $app->defaults($defaults)
     if $defaults;
   
   my $secret = $conf->{'mojo_secret'} || $conf->{'mojo_secrets'} || $conf->{'mojo'}{'secret'} || $conf->{'mojo'}{'secrets'} || $conf->{'шифры'} || [rand];
   $app->secrets($secret);
-
-  $app->mode($conf->{'mojo_mode'} || $conf->{'mojo'}{'mode'} || 'development'); # Файл лога уже не переключишь
-  $app->log->level( $conf->{'mojo_log_level'} || $conf->{'mojo'}{'log_level'} || 'debug');
+  
+  my $mode = $conf->{'mojo_mode'} || $conf->{'mojo'}{'mode'} || $conf->{'mojo.mode'};
+  $app->mode($mode) # Файл лога уже не переключишь
+    if $mode;
+  #~ $app->log->level( $conf->{'mojo_log_level'} || $conf->{'mojo'}{'log_level'} || 'debug');
+  my $log = $conf->{'mojo_log'} || $conf->{'mojo.log'} || $conf->{'mojo'}{'log'};
+  $app->log(Mojo::Log::Che->new(%$log))
+    if $log;
   #~ warn "Mode: ", $app->mode, "; log level: ", $app->log->level;
   
+  my $home = $app->home;
+  my $statics = $conf->{'mojo_static_paths'} || $conf->{'mojo.static.paths'} || $conf->{mojo}{static}{paths} || [];
+   #~ push @{$app->static->paths}, @{$paths} if $paths;
+  push @{$app->static->paths},  $home->rel_file($_) for @$statics;
+  
+  my $templates_paths = $conf->{'mojo_renderer_paths'} || $conf->{'mojo.renderer.paths'} || $conf->{mojo}{renderer}{paths} || [];
+  push @{$app->renderer->paths}, $home->rel_dir($_) for @$templates_paths;
+  
+  my $renderer_classes = $conf->{'mojo_renderer_classes'} || $conf->{'mojo.renderer.classes'} || $conf->{mojo}{renderer}{classes} || [];
+  push @{$app->renderer->classes}, $_ for grep ! load_class($_), @$renderer_classes;
+  
+  
   $app->сессия();
   $app->хазы();
   #~ $app->базы();
@@ -129,17 +139,7 @@ has sth => sub {
     }
   }
   
-  #~ my $sth_pos;
-  #~ while (my ($db, $arr) = each %$c_pos) {
-    #~ for my $item (@$arr) {
-      #~ $sth_pos ||= $app->_class('DBIx::POS::Sth');
-      #~ my $pos_module = $app->_class(ref $item eq 'ARRAY' ? shift @$item : $item);
-      #~ $sth->{$db}{$pos_module} = $sth_pos->new($dbh->{$db}, $pos_module->new(ref $item eq 'ARRAY' ? @$item : ()));
-      #~ $app->log->debug("Создан STH из POS модуля [$pos_module]");
-    #~ }
-  #~ }
-  
-  $sth;
+   $sth;
 };
 
   
@@ -149,9 +149,14 @@ sub хуки {# Хуки из конфига
   my $hooks = $conf->{'mojo_hooks'} || $conf->{'mojo'}{'hooks'} || $conf->{'хуки'}
      || return;
   while (my ($name, $sub) = each %$hooks) {
-  #~ map {
-    $app->hook($name => $sub);
-    $app->log->debug("Applied hook [$name] from config");
+    if (ref $sub eq 'ARRAY') {
+      $app->hook($name => $_)
+        for @$sub;
+    } else {
+      $app->hook($name => $sub);
+    }
+    
+    $app->log->debug(sprintf("Applied hook%s [%s] from config", ref $sub eq 'ARRAY' ? "s (@{[ scalar @$sub]})" : '', $name));
   }
 
 }
@@ -159,9 +164,13 @@ sub хуки {# Хуки из конфига
 sub сессия {
   my $app = shift;
   my $conf = $app->config;
-  my $session = $conf->{'mojo_session'} || $conf->{'mojo'}{'session'} || $conf->{'сессия'}
+  my $session = $conf->{'mojo_session'} || $conf->{'mojo.session'} || $conf->{'mojo'}{'session'} || $conf->{'сессия'}
     || return;
-  $app->sessions->cookie_name($session->{'cookie_name'});
+  $app->sessions->cookie_name($session->{'cookie_name'})
+    if $session->{'cookie_name'};
+  
+  $app->sessions->default_expiration($session->{'default_expiration'}) # set expiry
+    if defined $session->{'default_expiration'};
   
 }
 
@@ -206,24 +215,7 @@ sub спейсы {
   push @{$app->routes->namespaces}, @$ns;
 }
 
-#~ sub Mojolicious::Routes::is_hidden0000 {
-  #~ my ($self, $method) = @_;
-  #~ my $h = $self->{hiding} ||= {map { $_ => 1 } @{$self->hidden}};
-  ##return !!($h->{$method} || index($method, '_') == 0 || $method !~ /[a-z]/);
-  ##return !!($h->{$method} || index($method, '_') == 0);
-   #~ return !!($h->{$method} || $method =~ /^_/ || $method =~ /^[A-Z_]+$/);
-#~ }
-
-#~ sub _class {
-  #~ my $self = shift;
-  #~ my $class  = shift;
-  #~ my $e;
-  #~ $e = load_class($class)# success undef
-    #~ and die $e;
-  #~ return $class;
-#~ }
-
-1;
+our $VERSION = '0.031';
 
 =pod
 
@@ -235,6 +227,10 @@ sub спейсы {
 
 ¡ ¡ ¡ ALL GLORY TO GLORIA ! ! !
 
+=head1 VERSION
+
+0.031
+
 =head1 NAME
 
 Mojolicious::Che - Мой базовый модуль для приложений Mojolicious. Нужен только развернутый конфиг.
@@ -256,7 +252,9 @@ Mojolicious::Che - Мой базовый модуль для приложени
     # defaults =>
     # secrets =>
     # mode=>
-    # log_level =>
+    # log => {level=>...}
+    # static => {paths => [...]},
+    # renderer => {paths => [...], classes => [...], },
     # session =>
     # has =>
     # plugins=>
@@ -267,9 +265,11 @@ Mojolicious::Che - Мой базовый модуль для приложени
   # 'шифры' => [
   mojo_secrets => ['true 123 my app',],
   mojo_mode=> 'development',
-  mojo_log_level => 'debug',
+  mojo_log=>{level => 'error'},
+  mojo_static_paths => ["static"],
+  mojo_renderer_classes => ["Mojolicious::Foo::Fun"],
   # 'сессия' => 
-  mojo_session => {cookie_name => 'ELK'},
+  mojo_session => {cookie_name => 'EXX', default_expiration => 86400},
   
   # 'хазы' => 'Лет 500-700 назад был такой дикий степной торговый жадный народ ХАЗАРЫ. Столицей их "государства" был город Тьмутаракань, где-то на берегу моря Каспия. Потомки этих людей рассыпаны по странам России, Средней Азии, Европы. Есть мнение, что хазары присвоили себе название ЕВРЕИ, но это не те библейские кроткие евреи, а жадные потомки кроманьонцев'
   mojo_has => {
@@ -316,7 +316,8 @@ Mojolicious::Che - Мой базовый модуль для приложени
   # 'плугины'=> [
   mojo_plugins=>[ 
       ['Foo::Bar'],
-      ['Foo::Plugin' => sub {<...returns config data...>}],
+      [Foo::Bar::Plugin => opt1 => ..., opt2 => ...],
+      ['Foo::Plugin' => sub {<...returns config data list...>}],
   ],
   # 'хуки' => 
   mojo_hooks=>{
@@ -326,52 +327,54 @@ Mojolicious::Che - Мой базовый модуль для приложени
   namespaces => ['Space::Shattle'],
   # 'маршруты' => 
   routes => [
-    [get=>'/', to=> {cb=>sub{shift->render(format=>'txt', text=>'Hello!');},}],
+    [get=>'/', to=> {cb=>sub{shift->render(format=>'txt', text=>'Hello friend!');},}],
   ]
   };
 
-=head1 HAS's
+=head1 ATTRIBUTES
+
+B<Mojolicious::Che> inherits all attributes from L<Mojolicious> and implements the
+following new ones.
 
 =head2 dbh
 
 Set DBI handlers from config B<dbh> (или B<базы>)
 
+  my $dbh = $app->dbh->{main};
+
 =head2 sth
 
 Set prepared stattements from config B<sth> (или B<запросы>).
 
+  my $sth = $app->sth->{main}{foo};
+
 =head1 METHODS
 
-Mojolicious::Che inherits all methods from Mojolicious and implements the following new ones.
-All methods has nothing on input.
-
-=head2 new()
-
-Top-level method. Setup the B<defaults>, B<secrets>, B<mode>, B<log level> from app->config(). Then invoke all other metods in order below.
+B<Mojolicious::Che> inherits all methods from L<Mojolicious> and implements the following new ones.
 
 =head2 сессия()
 
-Session
+Session object config apply
 
 =head2 хазы()
 
-App has's
+Apply has's
 
 =head2 плугины()
 
-Plugins
+Apply plugins
 
 =head2 хуки()
 
-Hooks
+Apply hooks
 
 =head2 спейсы()
 
-Namespases
+Apply namespases
 
 =head2 маршруты()
 
-Routes
+Apply routes
 
 =head1 SEE ALSO
 
@@ -389,9 +392,11 @@ Please report any bugs or feature requests at L<https://github.com/mche/Mojolici
 
 =head1 COPYRIGHT
 
-Copyright 2016 Mikhail Che.
+Copyright 2016-2017 Mikhail Che.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
-=cut
\ No newline at end of file
+=cut
+
+1;
\ No newline at end of file
